import {Component, Input, OnChanges, OnDestroy, OnInit, SimpleChanges} from '@angular/core';
import {Product, ProductType} from "../../../model/product";
import {FormControl, FormGroup} from "@angular/forms";
import {Observable, Subscription} from "rxjs";
import {ProductsService} from "../../../services/products.service";
import {JobsService} from "../../../services/jobs.service";
import {faCubes, faEdit, faFileCode, faList} from "@fortawesome/free-solid-svg-icons";
import {AlertService} from "../../../services/alert.service";

@Component({
  selector: 'app-product-details',
  templateUrl: './product-details.component.html',
  styleUrls: ['./product-details.component.scss']
})
export class ProductDetailsComponent implements OnInit, OnChanges, OnDestroy {

  @Input() product: Product;

  type: ProductType;

  typeConfig: Observable<object>;
  productConfig: Observable<object>;
  mergedConfig: Observable<object>;

  showProductForm: boolean = false;
  jobFormGroup: FormGroup;

  public queues: Array<any> = [];
  public queues$: Subscription;

  public faEdit = faEdit;
  public faList = faList;
  public faCubes = faCubes;
  public faFileCode = faFileCode;

  constructor(private productService: ProductsService, private jobsService: JobsService, private alertService: AlertService) {
    this.jobFormGroup = new FormGroup({
      queue: new FormControl(),
      version: new FormControl()
    });
  }

  ngOnInit() {
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.hasOwnProperty('product')) {
      if (changes.product.currentValue.preRequisites.length > 0 &&
        changes.product.currentValue.preRequisites[0].requiredProduct.versions.length > 0) {
        let defaultVersion = changes.product.currentValue.preRequisites[0].requiredProduct.versions[0].versionNumber;
        for (let version of changes.product.currentValue.preRequisites[0].requiredProduct.versions) {
          if (version.status === 'QA_ACCEPTED' && defaultVersion < version.versionNumber) {
            defaultVersion = version.versionNumber;
          }
        }
        this.jobFormGroup.get('version').setValue(defaultVersion);
      }
      // product.preRequisites[0].requiredProduct.versions.reverse()
      this.productConfig = this.productService.getProductConfig(changes.product.currentValue.id, 'configurations');
      this.mergedConfig = this.productService.getProductConfig(changes.product.currentValue.id, 'merged');
      const productType = Product.getTypeFromName(changes.product.currentValue.typeName);
      if (productType) {
        this.type = productType;
        this.queues$ = this.productService.getPageInfo(productType.id).subscribe((queues: Array<string>) => {
          this.queues = queues;
          this.jobFormGroup.get('queue').setValue(queues[0]);
        });
        this.typeConfig = this.productService.getProductTypeConfig(productType.id);
      }
    }
  }

  createJob() {
    this.alertService.info('Creating Job...');
    let productId = this.product.id;
    let queue = this.jobFormGroup.get('queue').value;
    let version = parseInt(this.jobFormGroup.get('version').value);
    if (!version || isNaN(version)) {
      version = null;
    }
    this.jobsService.createJob(productId, queue, version).subscribe(response => {
      this.alertService.success('Job created');
    }, (error) => {
      this.alertService.error('Job creation failed');
    });
  }

  ngOnDestroy(): void {
    if (this.queues$) {
      this.queues$.unsubscribe();
    }
  }

}
