import {Injectable} from '@angular/core';
import {HttpClient, HttpParams} from "@angular/common/http";
import {Observable} from "rxjs";
import {ConfigurationService} from "../env/configuration.service";
import {map} from "rxjs/operators";
import {Tile, TileDefinition} from "../model/tile";
import {FiltersService} from "./filters.service";

@Injectable({
  providedIn: 'root'
})
export class TilesService {

  endPoint: string = '/services/minitiles/';

  constructor(
    private configService: ConfigurationService,
    private http: HttpClient,
    private filterService: FiltersService) {
  }

  public getTilesForEpoch(pattern: string, epoch: number): Observable<Array<Tile>> {
    let params = new HttpParams();

    const sortName = this.filterService.getCurrentSetting('TILE_SORT');
    const sortDir = this.filterService.getCurrentSetting('SORT_DIRECTION');

    params = params.append('pattern', pattern);
    params = params.append('columnNames', sortName);
    params = params.append('directions', sortDir);

    return this.http.get<Array<Tile>>(this.configService.config.url + this.endPoint + 'summary/epoch/' + epoch.toString(), {
      observe: 'response',
      params: params
    }).pipe(map(response => {
      return response.body;
    }));
  }

  public updateTileSummary(epoch: number): Observable<any> {
    return this.http.put<any>(this.configService.config.url + this.endPoint + 'summary/update?epoch=' + epoch.toString(), {}, {observe: 'response'})
      .pipe(map(response => {
      return response.body;
    }));
  }

  public getTileDefinition(id: number): Observable<TileDefinition> {
    return this.http.get<TileDefinition>(this.configService.config.url + this.endPoint + id + '/definitions',
      {observe: 'response'}).pipe(map(response => {
      return response.body;
    }));
  }
}
